@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/ed.defns.c,v 3.40 2004/08/04 17:12:27 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/ed.defns.c,v 3.51 2016/02/14 15:44:18 christos Exp $ */
 /*
  * ed.defns.c: Editor function definitions and initialization
  */
@@ -32,11 +32,11 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.defns.c,v 3.40 2004/08/04 17:12:27 christos Exp $")
+RCSID("$tcsh: ed.defns.c,v 3.51 2016/02/14 15:44:18 christos Exp $")
 
 #include "ed.h"
 
-static	void		ed_InitMetaBindings 	__P((void));
+static	void		ed_InitMetaBindings 	(void);
 
 PFCmd   CcFuncTbl[] = {		/* table of available commands */
     e_unassigned,
@@ -151,7 +151,7 @@ PFCmd   CcFuncTbl[] = {		/* table of available commands */
 #define         F_CASEUPPER     54
     e_lowercase,
 #define         F_CASELOWER     55
-    e_capitolcase,
+    e_capitalcase,
 #define         F_CASECAPITAL   56
     v_zero,
 #define		V_ZERO		57
@@ -279,8 +279,12 @@ PFCmd   CcFuncTbl[] = {		/* table of available commands */
 #define		F_PAGE_DOWN	118
     e_yank_pop,
 #define		F_YANK_POP	119
+    e_newline_hold,
+#define		F_NEWLINE_HOLD	120
+    e_newline_down_hist,
+#define		F_NEWLINE_DOWN_HIST	121
     0				/* DUMMY VALUE */
-#define		F_NUM_FNS	120
+#define		F_NUM_FNS	122
 
 };
 
@@ -493,7 +497,7 @@ KEYCMD  CcEmacsMap[] = {
     F_UNASSIGNED,		/* M-> */
     F_WHICH,			/* M-? */
     F_UNASSIGNED,		/* M-@ */
-    F_UNASSIGNED,		/* M-A */
+    F_NEWLINE_HOLD,		/* M-A */
     F_WORDBACK,			/* M-B */
     F_CASECAPITAL,		/* M-C */
     F_DELWORDNEXT,		/* M-D */
@@ -525,7 +529,7 @@ KEYCMD  CcEmacsMap[] = {
     F_UNASSIGNED,		/* M-^ */
     F_LAST_ITEM,		/* M-_ */
     F_UNASSIGNED,		/* M-` */
-    F_UNASSIGNED,		/* M-a */
+    F_NEWLINE_HOLD,		/* M-a */
     F_WORDBACK,			/* M-b */
     F_CASECAPITAL,		/* M-c */
     F_DELWORDNEXT,		/* M-d */
@@ -1120,7 +1124,7 @@ KEYCMD  CcViCmdMap[] = {
 
 
 void
-editinit()
+editinit(void)
 {
     struct KeyFuncs *f;
 
@@ -1128,7 +1132,7 @@ editinit()
     int i;
 
     for (i = 0; i < F_NUM_FUNCNAMES; i++)
-	xfree((ptr_t) FuncNames[i].desc);
+	xfree((ptr_t)(intptr_t)FuncNames[i].desc);
 #endif
 
     f = FuncNames;
@@ -1407,6 +1411,16 @@ editinit()
     f->func = F_NEWLINE;
     f->desc = CSAVS(3, 52, "Execute command");
 
+    f++;
+    f->name = "newline-and-hold";
+    f->func = F_NEWLINE_HOLD;
+    f->desc = CSAVS(3, 122, "Execute command and keep current line");
+
+    f++;
+    f->name = "newline-and-down-history";
+    f->func = F_NEWLINE_DOWN_HIST;
+    f->desc = CSAVS(3, 123, "Execute command and move to next history line");
+
     f++;
     f->name = "normalize-path";
     f->func = F_PATH_NORM;
@@ -1785,39 +1799,39 @@ editinit()
 
 #ifdef DEBUG_EDIT
 void
-CheckMaps()
+CheckMaps(void)
 {		/* check the size of the key maps */
-    int     c1 = (NT_NUM_KEYS * sizeof(KEYCMD));
+    size_t     c1 = NT_NUM_KEYS * sizeof(KEYCMD);
 
-    if ((sizeof(CcKeyMap)) != c1)
-	xprintf("CcKeyMap should be %d entries, but is %d.\r\n",
-		NT_NUM_KEYS, sizeof(CcKeyMap) / sizeof(KEYCMD)));
+    if (sizeof(CcKeyMap) != c1)
+	xprintf("CcKeyMap should be %u entries, but is %zu.\r\n",
+		NT_NUM_KEYS, sizeof(CcKeyMap) / sizeof(KEYCMD));
 
-    if ((sizeof(CcAltMap)) != c1)
-	xprintf("CcAltMap should be %d entries, but is %d.\r\n",
-		NT_NUM_KEYS, (sizeof(CcAltMap) / sizeof(KEYCMD)));
+    if (sizeof(CcAltMap) != c1)
+	xprintf("CcAltMap should be %u entries, but is %zu.\r\n",
+		NT_NUM_KEYS, sizeof(CcAltMap) / sizeof(KEYCMD));
 
-    if ((sizeof(CcEmacsMap)) != c1)
-	xprintf("CcEmacsMap should be %d entries, but is %d.\r\n",
-		NT_NUM_KEYS, (sizeof(CcEmacsMap) / sizeof(KEYCMD)));
+    if (sizeof(CcEmacsMap) != c1)
+	xprintf("CcEmacsMap should be %u entries, but is %zu.\r\n",
+		NT_NUM_KEYS, sizeof(CcEmacsMap) / sizeof(KEYCMD));
 
-    if ((sizeof(CcViMap)) != c1)
-	xprintf("CcViMap should be %d entries, but is %d.\r\n",
-		NT_NUM_KEYS, (sizeof(CcViMap) / sizeof(KEYCMD)));
+    if (sizeof(CcViMap) != c1)
+	xprintf("CcViMap should be %u entries, but is %zu.\r\n",
+		NT_NUM_KEYS, sizeof(CcViMap) / sizeof(KEYCMD));
 
-    if ((sizeof(CcViCmdMap)) != c1)
-	xprintf("CcViCmdMap should be %d entries, but is %d.\r\n",
-		NT_NUM_KEYS, (sizeof(CcViCmdMap) / sizeof(KEYCMD)));
+    if (sizeof(CcViCmdMap) != c1)
+	xprintf("CcViCmdMap should be %u entries, but is %zu.\r\n",
+		NT_NUM_KEYS, sizeof(CcViCmdMap) / sizeof(KEYCMD));
 }
 
 #endif
 
-bool    MapsAreInited = 0;
-bool    NLSMapsAreInited = 0;
-bool    NoNLSRebind;
+int    MapsAreInited = 0;
+int    NLSMapsAreInited = 0;
+int    NoNLSRebind;
 
 void
-ed_InitNLSMaps()
+ed_InitNLSMaps(void)
 {
     int i;
 
@@ -1826,15 +1840,15 @@ ed_InitNLSMaps()
     if (NoNLSRebind)
 	return;
     for (i = 0200; i <= 0377; i++) {
-	if (Isprint(i)) {
-	    CcKeyMap[i] = F_INSERT;
+	if (Isprint(CTL_ESC(i))) {
+	    CcKeyMap[CTL_ESC(i)] = F_INSERT;
 	}
     }
     NLSMapsAreInited = 1;
 }
 
 static void
-ed_InitMetaBindings()
+ed_InitMetaBindings(void)
 {
     Char    buf[3];
     int     i;
@@ -1842,13 +1856,13 @@ ed_InitMetaBindings()
     KEYCMD *map;
 
     map = CcKeyMap;
-    for (i = 0; i <= 0377 && CcKeyMap[i] != F_METANEXT; i++)
+    for (i = 0; i <= 0377 && CcKeyMap[CTL_ESC(i)] != F_METANEXT; i++)
 	continue;
     if (i > 0377) {
-	for (i = 0; i <= 0377 && CcAltMap[i] != F_METANEXT; i++)
+	for (i = 0; i <= 0377 && CcAltMap[CTL_ESC(i)] != F_METANEXT; i++)
 	    continue;
 	if (i > 0377) {
-	    i = CTL_ESC('\033');
+	    i = '\033';
 	    if (VImode)
 		map = CcAltMap;
 	}
@@ -1856,29 +1870,28 @@ ed_InitMetaBindings()
 	    map = CcAltMap;
 	}
     }
-    buf[0] = (Char) i;
+    buf[0] = (Char)CTL_ESC(i);
     buf[2] = 0;
     cstr.buf = buf;
     cstr.len = 2;
     for (i = 0200; i <= 0377; i++) {
-	if (map[i] != F_INSERT && map[i] != F_UNASSIGNED && map[i] != F_XKEY) {
-#ifdef IS_ASCII
-	    buf[1] = i & ASCII;
-#else
-	    buf[1] = _toebcdic[_toascii[i] & ASCII];
-#endif
-	    AddXkey(&cstr, XmapCmd((int) map[i]), XK_CMD);
+	if (map[CTL_ESC(i)] != F_INSERT && map[CTL_ESC(i)] != F_UNASSIGNED && map[CTL_ESC(i)] != F_XKEY) {
+	    buf[1] = CTL_ESC(i & ASCII);
+	    AddXkey(&cstr, XmapCmd((int) map[CTL_ESC(i)]), XK_CMD);
 	}
     }
     map[buf[0]] = F_XKEY;
 }
 
 void
-ed_InitVIMaps()
+ed_InitVIMaps(void)
 {
     int i;
 
     VImode = 1;
+    setNS(STRvimode);
+    update_wordchars();
+
     ResetXmap();
     for (i = 0; i < NT_NUM_KEYS; i++) {
 	CcKeyMap[i] = CcViMap[i];
@@ -1891,7 +1904,7 @@ ed_InitVIMaps()
 }
 
 void
-ed_InitEmacsMaps()
+ed_InitEmacsMaps(void)
 {
     int     i;
     Char    buf[3];
@@ -1900,6 +1913,10 @@ ed_InitEmacsMaps()
     cstr.len = 2;
 
     VImode = 0;
+    if (adrof(STRvimode))
+	unsetv(STRvimode);
+    update_wordchars();
+
     ResetXmap();
     for (i = 0; i < NT_NUM_KEYS; i++) {
 	CcKeyMap[i] = CcEmacsMap[i];
@@ -1934,7 +1951,7 @@ ed_InitEmacsMaps()
 }
 
 void
-ed_InitMaps()
+ed_InitMaps(void)
 {
     if (MapsAreInited)
 	return;
