@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.char.h,v 3.21 2004/08/04 17:12:28 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.char.h,v 3.37 2014/05/09 19:22:49 christos Exp $ */
 /*
  * sh.char.h: Table for spotting special characters quickly
  * 	      Makes for very obscure but efficient coding.
@@ -38,7 +38,11 @@
 #else
 # include <ctype.h>
 # ifdef WIDE_STRINGS
-#  include <wctype.h>
+#  ifdef HAVE_WCTYPE_H
+#   include <wctype.h>
+#  else
+#   include <wchar.h>
+#  endif
 # endif
 #endif
 
@@ -60,7 +64,9 @@ extern unsigned short _mbmap_big5[];
 extern unsigned short _mbmap_utf8[];
 /* VARIABLE Check str */
 /* same compiler require #define even not define DSPMBYTE */
+#undef	_MB1
 #define _MB1	0x0001
+#undef	_MB2
 #define _MB2	0x0002
 
 #ifndef NLS
@@ -68,73 +74,110 @@ extern tcshuc _cmap_lower[], _cmap_upper[];
 
 #endif
 
+#ifndef __QNXNTO__
+#undef	_QF
 #define	_QF	0x0001		/* '" (Forward quotes) */
+#undef	_QB
 #define	_QB	0x0002		/* ` (Backquote) */
+#undef	_SP
 #define	_SP	0x0004		/* space and tab */
+#else
+#undef	_XD
+#define	_XD	0x0001		/* As in <ctype.h> */
+#undef	_UP
+#define	_UP	0x0002		/* As in <ctype.h> */
+#undef	_SP
+#define	_SP	0x0004		/* As in <ctype.h> */
+#endif
+#undef	_NL
 #define	_NL	0x0008		/* \n */
+#undef	_META
 #define	_META	0x0010		/* lex meta characters, sp #'`";&<>()|\t\n */
+#undef	_GLOB
 #define	_GLOB	0x0020		/* glob characters, *?{[` */
+#undef	_ESC
 #define	_ESC	0x0040		/* \ */
+#undef	_DOL
 #define	_DOL	0x0080		/* $ */
+#undef	_DIG
 #define	_DIG  	0x0100		/* 0-9 */
+#undef	_LET
 #define	_LET  	0x0200		/* a-z, A-Z, _, or locale-specific */
+#ifndef __QNXNTO__
+#undef	_UP
 #define	_UP   	0x0400		/* A-Z, or locale-specific */
+#else
+#undef	_QF
+#define	_QF	0x0400		/* '" (Forward quotes) */
+#endif
+#undef	_DOW
 #define	_DOW  	0x0800		/* a-z, or locale-specific */
+#ifndef __QNXNTO__
+#undef	_XD
 #define	_XD 	0x1000		/* 0-9, a-f, A-F */
+#else
+#undef	_QB
+#define	_QB	0x1000		/* 0-9, a-f, A-F */
+#endif
+#undef	_CMD
 #define	_CMD	0x2000		/* lex end of command chars, ;&(|` */
+#undef	_CTR
 #define _CTR	0x4000		/* control */
+#undef	_PUN
 #define _PUN	0x8000		/* punctuation */
 
-#ifdef WIDE_STRINGS
-#  define ASC(ch) ch
-#  define CTL_ESC(ch) ch
-#  define cmap(c, bits)	\
-	(((c) & QUOTE) || (c) >= 0x0080 ? 0 : (_cmap[(tcshuc)(c)] & (bits)))
-#elif defined(SHORT_STRINGS) && defined(KANJI)
-# define ASC(ch) ch
-# define CTL_ESC(ch) ch
-# define cmap(c, bits) \
-	((((c) & QUOTE) || ((c & 0x80) && adrof(STRnokanji))) ? \
-	0 : (_cmap[(tcshuc)(c)] & (bits)))
-#else /* SHORT_STRINGS && KANJI */
-# ifdef IS_ASCII
-#  define ASC(ch) ch
-#  define CTL_ESC(ch) ch
-#  define cmap(c, bits)	\
-	(((c) & QUOTE) ? 0 : (_cmap[(tcshuc)(c)] & (bits)))
-# else /* IS_ASCII */
+#ifdef IS_ASCII
+# define ASC(ch) (ch)
+# define CTL_ESC(ch) (ch)
+#else
+# ifdef _OSD_POSIX
 /* "BS2000 OSD" is a POSIX on a main frame using a EBCDIC char set */
+#   include <ascii_ebcdic.h>
+# else
 /* "OS/390 USS" is a POSIX on a main frame using an IBM1047 char set */
-extern unsigned short _toascii[256];
-extern unsigned short _toebcdic[256];
+# endif
+  extern unsigned short _toascii[256];
+  extern unsigned short _toebcdic[256];
 
 /* mainly for comparisons if (ASC(ch)=='\177')... */
-#  define ASC(ch)     _toascii[(tcshuc)ch]
+#  define ASC(ch)     _toascii[(tcshuc)(ch)]
 
 /* Literal escapes ('\010') must be mapped to EBCDIC,
  * for C-Escapes   ('\b'), the compiler already does it.
  */
-#  define CTL_ESC(ch) _toebcdic[(tcshuc)ch]
+#  define CTL_ESC(ch) _toebcdic[(tcshuc)(ch)]
+#endif /*IS_ASCII*/
 
+#ifdef WIDE_STRINGS
+# define cmap(c, bits)	\
+    (((c) & QUOTE) || (unsigned int)(c) >= 0x0080 ? 0 : \
+	(_cmap[(tcshuc)ASC(c)] & (bits)))
+#elif defined(SHORT_STRINGS) && defined(KANJI)
 #  define cmap(c, bits)	\
-	(((c) & QUOTE) ? 0 : (_cmap[_toascii[(tcshuc)(c)]] & (bits)))
-# endif /* IS_ASCII */
+    ((((c) & QUOTE) || ((tcshuc)(ASC(c) & 0x80) && adrof(STRnokanji))) ? \
+	0 : (_cmap[(tcshuc)ASC(c)] & (bits)))
+#else /* SHORT_STRINGS && KANJI */
+# define cmap(c, bits)	\
+    (((c) & QUOTE) ? 0 : (_cmap[(tcshuc)ASC(c)] & (bits)))
 #endif /* SHORT_STRINGS && KANJI */
 
-#define isglob(c)	cmap(c, _GLOB)
-#define isspc(c)	cmap(c, _SP)
-#define ismeta(c)	cmap(c, _META)
-#define iscmdmeta(c)	cmap(c, _CMD)
+#define isglob(c)	cmap((c), _GLOB)
+#define isspc(c)	cmap((c), _SP)
+#define ismeta(c)	cmap((c), _META)
+#define iscmdmeta(c)	cmap((c), _CMD)
 #ifdef WIDE_STRINGS
 #define letter(c)	(((c) & QUOTE) ? 0 :  \
-			 (iswalpha((tcshuc) (c)) || (c) == '_'))
+			 (iswalpha((c)) || (c) == '_'))
 #define alnum(c)	(((c) & QUOTE) ? 0 :  \
-		         (iswalnum((tcshuc) (c)) || (c) == '_'))
+		         (iswalnum((c)) || (c) == '_'))
 #else
 #define letter(c)	(((Char)(c) & QUOTE) ? 0 :  \
-			 (isalpha((tcshuc) (c)) || (c) == '_'))
+			 ((isalpha((c)) && !(cmap((c), _PUN))) \
+			  || (c) == '_'))
 #define alnum(c)	(((Char)(c) & QUOTE) ? 0 :  \
-		         (isalnum((tcshuc) (c)) || (c) == '_'))
+		         ((isalnum((c)) && !(cmap((c), _PUN))) \
+			  || (c) == '_'))
+
 #endif
 
 #if defined(DSPMBYTE)
@@ -240,31 +283,25 @@ extern unsigned short _toebcdic[256];
 #  endif /* WINNT_NATIVE */
 # endif /* !NeXT */
 #else /* !NLS */
-# define Isspace(c)	cmap(c, _SP|_NL)
-# define Isdigit(c)	cmap(c, _DIG)
-# define Isalpha(c)	(cmap(c,_LET) && !(((c) & META) && AsciiOnly))
-# define Islower(c)	(cmap(c,_DOW) && !(((c) & META) && AsciiOnly))
-# define Isupper(c)	(cmap(c, _UP) && !(((c) & META) && AsciiOnly))
-# ifdef IS_ASCII
-#  define Tolower(c)	(_cmap_lower[(tcshuc)(c)])
-#  define Toupper(c)	(_cmap_upper[(tcshuc)(c)])
-# else
-/* "BS2000 OSD" is a POSIX on a main frame using a EBCDIC char set */
-#  define Tolower(c)	(_cmap_lower[_toascii[(tcshuc)(c)]])
-#  define Toupper(c)	(_cmap_upper[_toascii[(tcshuc)(c)]])
-# endif
-# define Isxdigit(c)	cmap(c, _XD)
-# define Isalnum(c)	(cmap(c, _DIG|_LET) && !(((Char)(c) & META) && AsciiOnly))
+# define Isspace(c)	cmap((c), _SP|_NL)
+# define Isdigit(c)	cmap((c), _DIG)
+# define Isalpha(c)	(cmap((c),_LET) && !(((c) & META) && AsciiOnly))
+# define Islower(c)	(cmap((c),_DOW) && !(((c) & META) && AsciiOnly))
+# define Isupper(c)	(cmap((c), _UP) && !(((c) & META) && AsciiOnly))
+# define Tolower(c)	(_cmap_lower[ASC(c)])
+# define Toupper(c)	(_cmap_upper[ASC(c)])
+# define Isxdigit(c)	cmap((c), _XD)
+# define Isalnum(c)	(cmap((c), _DIG|_LET) && !(((Char)(c) & META) && AsciiOnly))
 #if defined(DSPMBYTE)
-# define IscntrlM(c)	(cmap(c,_CTR) && !(((c) & META) && AsciiOnly))
+# define IscntrlM(c)	(cmap((c),_CTR) && !(((c) & META) && AsciiOnly))
 # define Iscntrl(c)	( (IscntrlM(c)) && !(_enable_mbdisp&&(IsmbyteU((c)))) )
-# define IsprintM(c)	(!cmap(c,_CTR) && !(((c) & META) && AsciiOnly))
+# define IsprintM(c)	(!cmap((c),_CTR) && !(((c) & META) && AsciiOnly))
 # define Isprint(c)	( (IsprintM(c)) || (_enable_mbdisp&&(IsmbyteU((c)))) )
 #else
-# define Iscntrl(c)	(cmap(c,_CTR) && !(((c) & META) && AsciiOnly))
-# define Isprint(c)	(!cmap(c,_CTR) && !(((c) & META) && AsciiOnly))
+# define Iscntrl(c)	(cmap((c),_CTR) && !(((c) & META) && AsciiOnly))
+# define Isprint(c)	(!cmap((c),_CTR) && !(((c) & META) && AsciiOnly))
 #endif /* !defined(DSPMBYTE) */
-# define Ispunct(c)	(cmap(c,_PUN) && !(((c) & META) && AsciiOnly))
+# define Ispunct(c)	(cmap((c),_PUN) && !(((c) & META) && AsciiOnly))
 
 #endif /* !NLS */
 
