@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.decls.h,v 3.61 2011/01/24 18:17:07 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.decls.h,v 3.68 2016/08/01 16:21:09 christos Exp $ */
 /*
  * sh.decls.h	 External declarations from sh*.c
  */
@@ -48,9 +48,9 @@ extern	void		  phup		(void);
 extern	void		  process	(int);
 extern	void		  untty		(void);
 #ifdef PROF
-extern	void		  done		(int);
+extern	void		  done		(int) __attribute__((__noreturn__));
 #else
-extern	void		  xexit		(int);
+extern	void		  xexit		(int) __attribute__((__noreturn__));
 #endif
 extern	int		  grabpgrp	(int, pid_t);
 
@@ -78,11 +78,12 @@ extern	void		  loaddirs	(Char *);
 extern	void		  Dfix		(struct command *);
 extern	Char		 *Dfix1		(Char *);
 extern	void		  heredoc	(Char *);
+extern  Char		 *randsuf	(void);
 
 /*
  * sh.err.c
  */
-extern	void		  reset		(void);
+extern	void		  reset		(void) __attribute__((__noreturn__));
 extern	void		  cleanup_push_internal(void *, void (*fn) (void *)
 #ifdef CLEANUP_DEBUG
 						, const char *, size_t
@@ -104,7 +105,9 @@ extern	void		  sigprocmask_cleanup(void *);
 extern	void		  xfree_indirect(void *);
 extern	void		  errinit	(void);
 extern	void		  seterror	(unsigned int, ...);
-extern	void		  stderror	(unsigned int, ...);
+extern	void		  fixerror	(void);
+extern	void		  stderror	(unsigned int, ...)
+    __attribute__((__noreturn__));
 
 /*
  * sh.exec.c
@@ -182,6 +185,7 @@ extern	void		  unalias	(Char **, struct command *);
 extern	void		  wfree		(void);
 extern	void		  dobuiltins	(Char **, struct command *);
 extern	void		  reexecute	(struct command *);
+extern  int		  getYN		(const char *);
 
 /*
  * sh.glob.c
@@ -219,6 +223,7 @@ extern	char		 *fmthist	(int, ptr_t);
 extern	void		  rechist	(Char *, int);
 extern	void		  loadhist	(Char *, int);
 extern	void		  displayHistStats(const char *);
+extern	void		  sethistory	(int);
 
 /*
  * sh.init.c
@@ -234,6 +239,7 @@ extern	void		  btell		(struct Ain *);
 extern	void		  btoeof	(void);
 extern	void		  copylex	(struct wordent *, struct wordent *);
 extern	Char		 *domod		(Char *, Char);
+extern	void		  initlex	(struct wordent *);
 extern	void		  freelex	(struct wordent *);
 extern	int		  lex		(struct wordent *);
 extern	void		  lex_cleanup	(void *);
@@ -241,6 +247,7 @@ extern	void		  prlex		(struct wordent *);
 extern	eChar		  readc		(int);
 extern	void		  settell	(void);
 extern	void		  unreadc	(Char);
+extern	ssize_t		  wide_read	(int, Char *, size_t, int);
 
 
 /*
@@ -276,7 +283,7 @@ extern	const Char	 *quote_meta	(struct Strbuf *, const Char *);
 extern	char		 *strnsave	(const char *, size_t);
 #endif
 extern	char		 *strsave	(const char *);
-extern	void		  udvar		(Char *);
+extern	void		  udvar		(Char *) __attribute__((__noreturn__));
 #ifndef POSIX
 extern  char   	  	 *strstr	(const char *, const char *);
 #endif /* !POSIX */
@@ -395,6 +402,7 @@ extern	void		  autoset_dspmbyte	(const Char *);
 #if defined(AUTOSET_KANJI)
 extern	void		  autoset_kanji	(void);
 #endif
+extern	void		  update_wordchars	(void);
 
 /*
  * sh.time.c
