@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/sh.dol.c,v 3.81 2010/05/17 19:36:45 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.dol.c,v 3.87 2014/08/13 23:39:34 amold Exp $ */
 /*
  * sh.dol.c: Variable substitutions
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: sh.dol.c,v 3.81 2010/05/17 19:36:45 christos Exp $")
+RCSID("$tcsh: sh.dol.c,v 3.87 2014/08/13 23:39:34 amold Exp $")
 
 /*
  * C shell
@@ -825,9 +825,8 @@ setDolp(Char *cp)
 		    (void) Strcpy(np + diff, rhsub);
 		    (void) Strcpy(np + diff + rhlen, dp + lhlen);
 
-		    dp = np + diff + 1;
 		    xfree(cp);
-		    cp = np;
+		    dp = cp = np;
 		    cp[--len] = '\0';
 		    didmod = 1;
 		    if (diff >= (ssize_t)len)
@@ -920,6 +919,19 @@ inheredoc_cleanup(void *dummy)
     inheredoc = 0;
 }
 
+Char *
+randsuf(void) {
+#ifndef WINNT_NATIVE
+	struct timeval tv;
+	(void) gettimeofday(&tv, NULL);
+	return putn((((tcsh_number_t)tv.tv_sec) ^ 
+	    ((tcsh_number_t)tv.tv_usec) ^
+	    ((tcsh_number_t)getpid())) & 0x00ffffff);
+#else
+    return putn(getpid());
+#endif
+}
+
 /*
  * Form a shell temporary file (in unit 0) from the words
  * of the shell input up to EOF or a line the same as "term".
@@ -936,46 +948,48 @@ heredoc(Char *term)
     Char *lbp, *obp, *mbp;
     Char  **vp;
     int    quoted;
+#ifdef HAVE_MKSTEMP
+    char   *tmp = short2str(shtemp);
+    char   *dot = strrchr(tmp, '.');
+
+    if (!dot)
+	stderror(ERR_NAME | ERR_NOMATCH);
+    strcpy(dot, TMP_TEMPLATE);
+
+    xclose(0);
+    if (mkstemp(tmp) == -1)
+	stderror(ERR_SYSTEM, tmp, strerror(errno));
+#else /* !HAVE_MKSTEMP */
     char   *tmp;
-#ifndef WINNT_NATIVE
-    struct timeval tv;
+# ifndef WINNT_NATIVE
 
 again:
-#endif /* WINNT_NATIVE */
+# endif /* WINNT_NATIVE */
     tmp = short2str(shtemp);
-#ifndef O_CREAT
-# define O_CREAT 0
+# if O_CREAT == 0
     if (xcreat(tmp, 0600) < 0)
 	stderror(ERR_SYSTEM, tmp, strerror(errno));
-#endif
+# endif
     xclose(0);
-#ifndef O_TEMPORARY
-# define O_TEMPORARY 0
-#endif
-#ifndef O_EXCL
-# define O_EXCL 0
-#endif
     if (xopen(tmp, O_RDWR|O_CREAT|O_EXCL|O_TEMPORARY|O_LARGEFILE, 0600) ==
 	-1) {
 	int oerrno = errno;
-#ifndef WINNT_NATIVE
+# ifndef WINNT_NATIVE
 	if (errno == EEXIST) {
 	    if (unlink(tmp) == -1) {
-		(void) gettimeofday(&tv, NULL);
 		xfree(shtemp);
-		mbp = putn((((tcsh_number_t)tv.tv_sec) ^ 
-		    ((tcsh_number_t)tv.tv_usec) ^
-		    ((tcsh_number_t)getpid())) & 0x00ffffff);
+		mbp = randsuf();
 		shtemp = Strspl(STRtmpsh, mbp);
 		xfree(mbp);
 	    }
 	    goto again;
 	}
-#endif /* WINNT_NATIVE */
+# endif /* WINNT_NATIVE */
 	(void) unlink(tmp);
 	errno = oerrno;
  	stderror(ERR_SYSTEM, tmp, strerror(errno));
     }
+#endif /* HAVE_MKSTEMP */
     (void) unlink(tmp);		/* 0 0 inode! */
     Dv[0] = term;
     Dv[1] = NULL;
@@ -1010,7 +1024,7 @@ again:
 
 	/* Catch EOF in the middle of a line. */
 	if (c == CHAR_ERR && lbuf.len != 0)
-	    stderror(ERR_EOF);
+	    c = '\n';
 
 	/*
 	 * Check for EOF or compare to terminator -- before expansion
