@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.time.c,v 3.28 2005/03/03 16:49:16 kim Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.time.c,v 3.37 2016/07/09 00:45:29 christos Exp $ */
 /*
  * sh.time.c: Shell time keeping and printing.
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.time.c,v 3.28 2005/03/03 16:49:16 kim Exp $")
+RCSID("$tcsh: sh.time.c,v 3.37 2016/07/09 00:45:29 christos Exp $")
 
 #ifdef SUNOS4
 # include <machine/param.h>
@@ -52,17 +52,17 @@ struct tms times0;
 
 #if !defined(BSDTIMES) && !defined(_SEQUENT_)
 # ifdef	POSIX
-static	void	pdtimet	__P((clock_t, clock_t));
+static	void	pdtimet	(clock_t, clock_t);
 # else /* ! POSIX */
-static	void	pdtimet	__P((time_t, time_t));
+static	void	pdtimet	(time_t, time_t);
 # endif	/* ! POSIX */
 #else /* BSDTIMES || _SEQUENT_ */
-static	void	tvadd	__P((timeval_t *, timeval_t *));
-static	void	pdeltat	__P((timeval_t *, timeval_t *));
+static	void	tvadd	(timeval_t *, timeval_t *);
+static	void	pdeltat	(timeval_t *, timeval_t *);
 #endif /* BSDTIMES || _SEQUENT_	*/
 
 void
-settimes()
+settimes(void)
 {
 #ifdef BSDTIMES
     struct sysrusage ruch;
@@ -83,12 +83,7 @@ settimes()
     ruadd(&ru0,	&ruch);
 # else	/* _SEQUENT_ */
     seconds0 = time(NULL);
-#  ifndef COHERENT
     time0 = times(&times0);
-#  else	/* !COHERENT */
-    time0 = HZ * seconds0;
-    times(&times0);
-#  endif /* !COHERENT */
     times0.tms_stime +=	times0.tms_cstime;
     times0.tms_utime +=	times0.tms_cutime;
     times0.tms_cstime =	0;
@@ -103,9 +98,7 @@ settimes()
  */
 /*ARGSUSED*/
 void
-dotime(v, c)
-    Char **v;
-    struct command *c;
+dotime(Char **v, struct command *c)
 {
 #ifdef BSDTIMES
     timeval_t timedol;
@@ -137,12 +130,7 @@ dotime(v, c)
 
     struct tms times_dol;
 
-#ifndef	COHERENT
     timedol = times(&times_dol);
-#else
-    timedol = HZ * time(NULL);
-    times(&times_dol);
-#endif
     times_dol.tms_stime	+= times_dol.tms_cstime;
     times_dol.tms_utime	+= times_dol.tms_cutime;
     times_dol.tms_cstime = 0;
@@ -159,9 +147,7 @@ dotime(v, c)
  */
 /*ARGSUSED*/
 void
-donice(v, c)
-    Char **v;
-    struct command *c;
+donice(Char **v, struct command *c)
 {
     Char *cp;
     int	    nval = 0;
@@ -172,18 +158,17 @@ donice(v, c)
 	nval = 4;
     else if (*v	== 0 &&	any("+-", cp[0]))
 	nval = getn(cp);
-#ifdef HAVE_SETPRIORITY
+#if defined(HAVE_SETPRIORITY) && defined(PRIO_PROCESS)
     if (setpriority(PRIO_PROCESS, 0, nval) == -1 && errno)
 	stderror(ERR_SYSTEM, "setpriority", strerror(errno));
-#else /* !HAVE_SETPRIORITY */
+#else /* !HAVE_SETPRIORITY || !PRIO_PROCESS */
     (void) nice(nval);
-#endif /* HAVE_SETPRIORITY */
+#endif /* HAVE_SETPRIORITY && PRIO_PROCESS */
 }
 
 #ifdef BSDTIMES
 void
-ruadd(ru, ru2)
-    struct sysrusage *ru,	*ru2;
+ruadd(struct sysrusage *ru, struct sysrusage *ru2)
 {
     tvadd(&ru->ru_utime, &ru2->ru_utime);
     tvadd(&ru->ru_stime, &ru2->ru_stime);
@@ -218,8 +203,7 @@ ruadd(ru, ru2)
 #else /* BSDTIMES */
 # ifdef	_SEQUENT_
 void
-ruadd(ru, ru2)
-    struct process_stats *ru, *ru2;
+ruadd(struct process_stats *ru, struct process_stats *ru2)
 {
     tvadd(&ru->ps_utime, &ru2->ps_utime);
     tvadd(&ru->ps_stime, &ru2->ps_stime);
@@ -290,32 +274,26 @@ ruadd(ru, ru2)
 #endif /* SUNOS4 */
 
 void
-prusage(r0, r1,	e, b)
-    struct sysrusage *r0,	*r1;
-    timeval_t *e, *b;
+prusage(struct sysrusage *r0, struct sysrusage *r1, timeval_t *e, timeval_t *b)
 
 #else /* BSDTIMES */
 # ifdef	_SEQUENT_
 void
-prusage(r0, r1,	e, b)
-    struct process_stats *r0, *r1;
-    timeval_t *e, *b;
+prusage(struct process_stats *r0, struct process_stats *r1, timeval_t e,
+	timeval_t b)
 
 # else /* _SEQUENT_ */
-void
-prusage(bs, es,	e, b)
-    struct tms *bs, *es;
-
 #  ifndef POSIX
-    time_t  e, b;
-
+void
+prusage(struct tms *bs, struct tms *es, time_t e, time_t b)
 #  else	/* POSIX */
-    clock_t e, b;
-
+void
+prusage(struct tms *bs, struct tms *es, clock_t e, clock_t b)
 #  endif /* POSIX */
 # endif	/* _SEQUENT_ */
 #endif /* BSDTIMES */
 {
+    int ohaderr = haderr;
 #ifdef BSDTIMES
     time_t t =
     (r1->ru_utime.tv_sec - r0->ru_utime.tv_sec)	* 100 +
@@ -357,12 +335,14 @@ prusage(bs, es,	e, b)
     ((e->tv_sec	- b->tv_sec) * 100 + (e->tv_usec - b->tv_usec) / 10000);
 
     cp = "%Uu %Ss %E %P	%X+%Dk %I+%Oio %Fpf+%Ww";
+    haderr = 0;
 #else /* !BSDTIMES */
 # ifdef	_SEQUENT_
     int	    ms = (int)
     ((e->tv_sec	- b->tv_sec) * 100 + (e->tv_usec - b->tv_usec) / 10000);
 
     cp = "%Uu %Ss %E %P	%I+%Oio	%Fpf+%Ww";
+    haderr = 0;
 # else /* !_SEQUENT_ */
 #  ifndef POSIX
     time_t ms = ((time_t)((e - b) / HZ) * 100) +
@@ -373,6 +353,7 @@ prusage(bs, es,	e, b)
 #  endif /* POSIX */
 
     cp = "%Uu %Ss %E %P";
+    haderr = 0;
 
     /*
      * the tms stuff is	not very precise, so we	fudge it.
@@ -389,11 +370,11 @@ prusage(bs, es,	e, b)
 #endif /* !BSDTIMES */
 #ifdef TDEBUG
     xprintf("es->tms_utime %lu bs->tms_utime %lu\n",
-	    es->tms_utime, bs->tms_utime);
+	    (unsigned long)es->tms_utime, (unsigned long)bs->tms_utime);
     xprintf("es->tms_stime %lu bs->tms_stime %lu\n",
-	    es->tms_stime, bs->tms_stime);
-    xprintf("ms	%lu e %lu b %lu\n", ms,	e, b);
-    xprintf("t %lu\n", t);
+	    (unsigned long)es->tms_stime, (unsigned long)bs->tms_stime);
+    xprintf("ms	%llu e %p b %p\n", (unsigned long long)ms, e, b);
+    xprintf("t %llu\n", (unsigned long long)t);
 #endif /* TDEBUG */
 
     if (vp && vp->vec && vp->vec[0] && vp->vec[1])
@@ -477,7 +458,6 @@ prusage(bs, es,	e, b)
 				 (long long)r0->ru_ixrss) /
 			 (long long)t);
 		xprintf("%lu", (unsigned long)memtmp);
-			
 		break;
 
 	    case 'D':		/* (average) unshared data size	*/
@@ -504,8 +484,8 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", t == 0 ?	0L :
-			IADJUST(r1->ru_ixrss - r0->ru_ixrss) / t);
+		xprintf("%lld", (long long)(t == 0 ?	0L :
+			IADJUST(r1->ru_ixrss - r0->ru_ixrss) / t));
 #endif
 		break;
 
@@ -513,9 +493,9 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", t == 0 ?	0L :
+		xprintf("%lld", (long long)(t == 0 ?	0L :
 			IADJUST(r1->ru_idrss + r1->ru_isrss -
-				(r0->ru_idrss +	r0->ru_isrss)) / t);
+				(r0->ru_idrss +	r0->ru_isrss)) / t));
 #endif
 		break;
 
@@ -523,23 +503,23 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", t == 0 ?	0L :
+		xprintf("%lld", (long long)(t == 0 ? 0L :
 			IADJUST((r1->ru_ixrss +	r1->ru_isrss + r1->ru_idrss) -
-			   (r0->ru_ixrss + r0->ru_idrss	+ r0->ru_isrss)) / t);
+			   (r0->ru_ixrss + r0->ru_idrss	+ r0->ru_isrss)) / t));
 #endif
 		break;
 #endif /* convex */
 	    case 'M':		/* max.	Resident Set Size */
 #ifdef SUNOS4
-		xprintf("%ld", pagetok(r1->ru_maxrss));
+		xprintf("%ld", (long)pagetok(r1->ru_maxrss));
 #else
 # ifdef	convex
-		xprintf("%ld", r1->ru_maxrss * 4L);
+		xprintf("%ld", (long)(r1->ru_maxrss * 4L));
 # else /* !convex */
 #  ifdef _OSD_POSIX
 		xprintf("0",0);
 #  else
-		xprintf("%ld", r1->ru_maxrss / 2L);
+		xprintf("%ld", (long)r1->ru_maxrss);
 #  endif
 # endif	/* convex */
 #endif /* SUNOS4 */
@@ -549,7 +529,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_majflt - r0->ru_majflt);
+		xprintf("%ld", (long)(r1->ru_majflt - r0->ru_majflt));
 #endif
 		break;
 
@@ -557,7 +537,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_minflt - r0->ru_minflt);
+		xprintf("%ld", (long)(r1->ru_minflt - r0->ru_minflt));
 #endif
 		break;
 
@@ -565,7 +545,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_inblock -	r0->ru_inblock);
+		xprintf("%ld", (long)(r1->ru_inblock - r0->ru_inblock));
 #endif
 		break;
 
@@ -573,7 +553,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_oublock -	r0->ru_oublock);
+		xprintf("%ld", (long)(r1->ru_oublock - r0->ru_oublock));
 #endif
 		break;
 
@@ -591,7 +571,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_msgrcv - r0->ru_msgrcv);
+		xprintf("%ld", (long)(r1->ru_msgrcv - r0->ru_msgrcv));
 #endif
 		break;
 
@@ -599,7 +579,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_msgsnd - r0->ru_msgsnd);
+		xprintf("%ld", (long)(r1->ru_msgsnd - r0->ru_msgsnd));
 #endif
 		break;
 
@@ -607,7 +587,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_nsignals - r0->ru_nsignals);
+		xprintf("%ld", (long)(r1->ru_nsignals - r0->ru_nsignals));
 #endif
 		break;
 
@@ -615,7 +595,7 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_nvcsw - r0->ru_nvcsw);
+		xprintf("%ld", (long)(r1->ru_nvcsw - r0->ru_nvcsw));
 #endif
 		break;
 
@@ -623,42 +603,42 @@ prusage(bs, es,	e, b)
 #ifdef _OSD_POSIX
 		xprintf("0",0);
 #else
-		xprintf("%ld", r1->ru_nivcsw - r0->ru_nivcsw);
+		xprintf("%ld", (long)(r1->ru_nivcsw - r0->ru_nivcsw));
 #endif
 		break;
 #else /* BSDTIMES */
 # ifdef	_SEQUENT_
 	    case 'W':		/* number of swaps */
 		i = r1->ps_swap	- r0->ps_swap;
-		xprintf("%ld", i);
+		xprintf("%ld", (long)i);
 		break;
 
 	    case 'M':
-		xprintf("%ld", r1->ps_maxrss / 2);
+		xprintf("%ld", (long)r1->ps_maxrss);
 		break;
 
 	    case 'F':
-		xprintf("%ld", r1->ps_pagein - r0->ps_pagein);
+		xprintf("%ld", (long)(r1->ps_pagein - r0->ps_pagein));
 		break;
 
 	    case 'R':
-		xprintf("%ld", r1->ps_reclaim -	r0->ps_reclaim);
+		xprintf("%ld", (long)(r1->ps_reclaim -	r0->ps_reclaim));
 		break;
 
 	    case 'I':
-		xprintf("%ld", r1->ps_bread - r0->ps_bread);
+		xprintf("%ld", (long)(r1->ps_bread - r0->ps_bread));
 		break;
 
 	    case 'O':
-		xprintf("%ld", r1->ps_bwrite - r0->ps_bwrite);
+		xprintf("%ld", (long)(r1->ps_bwrite - r0->ps_bwrite));
 		break;
 
 	    case 'k':
-		xprintf("%ld", r1->ps_signal - r0->ps_signal);
+		xprintf("%ld", (long)(r1->ps_signal - r0->ps_signal));
 		break;
 
 	    case 'w':
-		xprintf("%ld", r1->ps_volcsw - r0->ps_volcsw);
+		xprintf("%ld", (long)(r1->ps_volcsw - r0->ps_volcsw));
 		break;
 
 	    case 'c':
@@ -666,35 +646,35 @@ prusage(bs, es,	e, b)
 		break;
 
 	    case 'Z':
-		xprintf("%ld", r1->ps_zerofill - r0->ps_zerofill);
+		xprintf("%ld", (long)(r1->ps_zerofill - r0->ps_zerofill));
 		break;
 
 	    case 'i':
-		xprintf("%ld", r1->ps_pffincr -	r0->ps_pffincr);
+		xprintf("%ld", (long)(r1->ps_pffincr - r0->ps_pffincr));
 		break;
 
 	    case 'd':
-		xprintf("%ld", r1->ps_pffdecr -	r0->ps_pffdecr);
+		xprintf("%ld", (long)(r1->ps_pffdecr - r0->ps_pffdecr));
 		break;
 
 	    case 'Y':
-		xprintf("%ld", r1->ps_syscall -	r0->ps_syscall);
+		xprintf("%ld", (long)(r1->ps_syscall - r0->ps_syscall));
 		break;
 
 	    case 'l':
-		xprintf("%ld", r1->ps_lread - r0->ps_lread);
+		xprintf("%ld", (long)(r1->ps_lread - r0->ps_lread));
 		break;
 
 	    case 'm':
-		xprintf("%ld", r1->ps_lwrite - r0->ps_lwrite);
+		xprintf("%ld", (long)(r1->ps_lwrite - r0->ps_lwrite));
 		break;
 
 	    case 'p':
-		xprintf("%ld", r1->ps_phread - r0->ps_phread);
+		xprintf("%ld", (long)(r1->ps_phread - r0->ps_phread));
 		break;
 
 	    case 'q':
-		xprintf("%ld", r1->ps_phwrite -	r0->ps_phwrite);
+		xprintf("%ld", (long)(r1->ps_phwrite - r0->ps_phwrite));
 		break;
 # endif	/* _SEQUENT_ */
 #endif /* BSDTIMES */
@@ -702,22 +682,21 @@ prusage(bs, es,	e, b)
 		break;
 	    }
     xputchar('\n');
+    haderr = ohaderr;
 }
 
 #if defined(BSDTIMES) || defined(_SEQUENT_)
 static void
-pdeltat(t1, t0)
-    timeval_t *t1, *t0;
+pdeltat(timeval_t *t1, timeval_t *t0)
 {
     timeval_t td;
 
     tvsub(&td, t1, t0);
-    xprintf("%ld.%03ld", td.tv_sec, td.tv_usec / 1000L);
+    xprintf("%lld.%03ld", (long long)td.tv_sec, (long)td.tv_usec / 1000L);
 }
 
 static void
-tvadd(tsum, t0)
-    timeval_t *tsum, *t0;
+tvadd(timeval_t *tsum, timeval_t *t0)
 {
 
     tsum->tv_sec += t0->tv_sec;
@@ -727,8 +706,7 @@ tvadd(tsum, t0)
 }
 
 void
-tvsub(tdiff, t1, t0)
-    timeval_t *tdiff, *t1, *t0;
+tvsub(timeval_t *tdiff, timeval_t *t1, timeval_t *t0)
 {
 
     tdiff->tv_sec = t1->tv_sec - t0->tv_sec;
@@ -739,12 +717,11 @@ tvsub(tdiff, t1, t0)
 
 #else /* !BSDTIMES && !_SEQUENT_ */
 static void
-pdtimet(eval, bval)
 #ifndef	POSIX
-    time_t  eval, bval;
+pdtimet(time_t eval, time_t bval)
 
 #else /* POSIX */
-    clock_t eval, bval;
+pdtimet(clock_t eval, clock_t bval)
 
 #endif /* POSIX	*/
 {
@@ -762,6 +739,7 @@ pdtimet(eval, bval)
     val	= (eval	- bval)	* 100 /	clk_tck;
 #endif /* POSIX	*/
 
-    xprintf("%ld.%02ld", val / 100, val	- (val / 100 * 100));
+    xprintf("%lld.%02ld", (long long)(val / 100),
+	(long long)(val - (val / 100 * 100)));
 }
 #endif /* BSDTIMES || _SEQUENT_	*/
