@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tc.decls.h,v 3.64 2008/05/14 20:10:30 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/tc.decls.h,v 3.66 2012/06/21 18:49:11 christos Exp $ */
 /*
  * tc.decls.h: Function declarations from all the tcsh modules
  */
@@ -226,15 +226,15 @@ extern	int		  getv		(Char *);
 #ifndef __GNUC__
 #define __attribute__(a)
 #endif
-extern	pret_t		  xprintf	(const char *, ...)
+extern	int		  xprintf	(const char *, ...)
     __attribute__((__format__(__printf__, 1, 2)));
-extern	pret_t		  xsnprintf	(char *, size_t, const char *, ...)
+extern	int		  xsnprintf	(char *, size_t, const char *, ...)
     __attribute__((__format__(__printf__, 3, 4)));
 extern	char		 *xasprintf	(const char *, ...)
     __attribute__((__format__(__printf__, 1, 2)));
-extern	pret_t		  xvprintf	(const char *, va_list)
+extern	int		  xvprintf	(const char *, va_list)
     __attribute__((__format__(__printf__, 1, 0)));
-extern	pret_t		  xvsnprintf	(char *, size_t, const char *, va_list)
+extern	int		  xvsnprintf	(char *, size_t, const char *, va_list)
     __attribute__((__format__(__printf__, 3, 0)));
 extern	char		 *xvasprintf	(const char *, va_list)
     __attribute__((__format__(__printf__, 1, 0)));
@@ -259,9 +259,9 @@ extern	void		  sched_run	(void);
  * tc.str.c:
  */
 #ifdef WIDE_STRINGS
-extern	size_t		  one_mbtowc	(wchar_t *, const char *, size_t);
-extern	size_t		  one_wctomb	(char *, wchar_t);
-extern  int		  rt_mbtowc	(wchar_t *, const char *, size_t);
+extern	size_t		  one_mbtowc	(Char *, const char *, size_t);
+extern	size_t		  one_wctomb	(char *, Char);
+extern  int		  rt_mbtowc	(Char *, const char *, size_t);
 #else
 #define one_mbtowc(PWC, S, N) \
 	((void)(N), *(PWC) = (unsigned char)*(S), (size_t)1)
