@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tc.os.c,v 3.72 2011/01/25 13:58:19 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/tc.os.c,v 3.73 2016/04/16 15:44:18 christos Exp $ */
 /*
  * tc.os.c: OS Dependent builtin functions
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: tc.os.c,v 3.72 2011/01/25 13:58:19 christos Exp $")
+RCSID("$tcsh: tc.os.c,v 3.73 2016/04/16 15:44:18 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -1585,28 +1585,6 @@ isapad(void)
 }
 #endif
 
-#ifdef __ANDROID__
-#include <stdio.h>
-/* Android (<= 2.1?) has an incomplete ttyname implementation. */
-char *
-ttyname(int fd)
-{
-    char path[64];
-    ssize_t siz;
-    static char ttyname[32];
-
-    if (!isatty(fd))
-	return NULL;
-
-    (void)snprintf(path, sizeof(path), "/proc/self/fd/%d", fd);
-    siz = readlink(path, ttyname, sizeof(ttyname));
-    if (siz < 0 || siz == sizeof(ttyname))
-	return NULL;
-    ttyname[siz] = '\0';
-    return ttyname;
-}
-#endif /* __ANDROID__ */
-
 #if defined(__CYGWIN__) && !defined(NO_CRYPT)
 #undef CHAR		/* Collides with Win32 API */
 #define WIN32_LEAN_AND_MEAN
