@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tw.comp.c,v 1.44 2014/07/24 10:50:15 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/tw.comp.c,v 1.45 2015/09/30 13:28:02 christos Exp $ */
 /*
  * tw.comp.c: File completion builtin
  */
@@ -32,7 +32,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: tw.comp.c,v 1.44 2014/07/24 10:50:15 christos Exp $")
+RCSID("$tcsh: tw.comp.c,v 1.45 2015/09/30 13:28:02 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -251,7 +251,7 @@ static int
 tw_match(const Char *str, const Char *pat, int exact)
 {
     const Char *estr;
-    int rv = exact ? Gmatch(str, pat) : Gnmatch(str, pat, &estr);
+    int rv = exact ? Gmatch(estr = str, pat) : Gnmatch(str, pat, &estr);
 #ifdef TDEBUG
     xprintf("G%smatch(%s, ", exact ? "" : "n", short2str(str));
     xprintf("%s, ", short2str(pat));
