@@ -1,4 +1,4 @@
-/* $Header: /p/tcsh/cvsroot/tcsh/tw.parse.c,v 3.131 2011/01/24 18:50:44 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/tw.parse.c,v 3.139 2015/10/16 14:59:56 christos Exp $ */
 /*
  * tw.parse.c: Everyone has taken a shot in this futile effort to
  *	       lexically analyze a csh line... Well we cannot good
@@ -35,7 +35,7 @@
  */
 #include "sh.h"
 
-RCSID("$tcsh: tw.parse.c,v 3.131 2011/01/24 18:50:44 christos Exp $")
+RCSID("$tcsh: tw.parse.c,v 3.139 2015/10/16 14:59:56 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -124,7 +124,8 @@ static  int      tw_collect_items	(COMMAND, int, struct Strbuf *,
 static  int      tw_collect		(COMMAND, int, struct Strbuf *,
 					 struct Strbuf *, Char *, Char *, int,
 					 DIR *);
-static	Char 	 tw_suffix		(int, const Char *, Char *);
+static	Char 	 tw_suffix		(int, struct Strbuf *,const Char *,
+					 Char *);
 static	void 	 tw_fixword		(int, struct Strbuf *, Char *, Char *);
 static	void	 tw_list_items		(int, int, int);
 static 	void	 add_scroll_tab		(Char *);
@@ -358,7 +359,7 @@ tenematch(Char *inputline, int num_read, COMMAND command)
 	 * Trying to spell can hang when we have NFS mounted hung
 	 * volumes.
 	 */
-	if ((looking == TW_COMMAND) && ABSOLUTEP(wordbuf.s)) {
+	if ((looking == TW_COMMAND) && Strchr(wordbuf.s, '/') != NULL) {
 	    if (executable(NULL, wordbuf.s, 0)) {
 		cleanup_until(&wordbuf);
 		search_ret = 0;
@@ -617,7 +618,11 @@ insert_meta(const Char *cp, const Char *cpend, const Char *word,
 	    break;
 
 	wq = w & QUOTE;
-	w &= ~QUOTE;
+#if INVALID_BYTE != 0
+	/* add checking INVALID_BYTE for FIX UTF32 */
+	if ((w & INVALID_BYTE) != INVALID_BYTE)		/* w < INVALID_BYTE */
+#endif
+	    w &= ~QUOTE;
 
 	if (cmap(w, _ESC | _QF))
 	    wq = QUOTE;		/* quotes are always quoted */
@@ -1232,9 +1237,10 @@ tw_collect_items(COMMAND command, int looking, struct Strbuf *exp_dir,
  */
 /*ARGSUSED*/
 static Char
-tw_suffix(int looking, const Char *exp_dir, Char *exp_name)
+tw_suffix(int looking, struct Strbuf *word, const Char *exp_dir, Char *exp_name)
 {
     Char *ptr;
+    Char *dol;
     struct varent *vp;
 
     (void) strip(exp_name);
@@ -1256,6 +1262,10 @@ tw_suffix(int looking, const Char *exp_dir, Char *exp_name)
 	else if ((ptr = tgetenv(exp_name)) == NULL || *ptr == '\0')
 	    return ' ';
 
+	if ((dol = Strrchr(word->s, '$')) != 0 && 
+	    dol[1] == '{' && Strchr(dol, '}') == NULL)
+	  return '}';
+
 	return isadirectory(exp_dir, ptr) ? '/' : ' ';
 
 
@@ -1303,6 +1313,7 @@ tw_fixword(int looking, struct Strbuf *word, Char *dir, Char *exp_name)
 
     case TW_VARIABLE:
 	if ((ptr = Strrchr(word->s, '$')) != NULL) {
+	    if (ptr[1] == '{') ptr++;
 	    word->len = ptr + 1 - word->s; /* Delete after the dollar */
 	} else
 	    word->len = 0;
@@ -1538,8 +1549,10 @@ t_search(struct Strbuf *word, COMMAND command, int looking, int list_max,
 	gpat = 0;	/* Override pattern mechanism */
     }
     else if ((target = Strrchr(name, '$')) != 0 && 
+	     (target[1] != '{' || Strchr(target, '}') == NULL) &&
 	     (Strchr(name, '/') == NULL)) {
 	target++;
+	if (target[0] == '{') target++;
 	looking = TW_VARIABLE;
 	gpat = 0;	/* Override pattern mechanism */
     }
@@ -1782,7 +1795,7 @@ t_search(struct Strbuf *word, COMMAND command, int looking, int list_max,
 	    switch (suf) {
 	    case 0: 	/* Automatic suffix */
 		Strbuf_append1(word,
-			       tw_suffix(looking, exp_dir.s, exp_name.s));
+			       tw_suffix(looking, word, exp_dir.s, exp_name.s));
 		break;
 
 	    case CHAR_ERR:	/* No suffix */
